<?php

namespace App\Contract;

use App\Service\PricingStrategy;
use App\Service\TrivialPricingStrategy;
use Symfony\Component\Uid\Uuid;

class Order
{
    public function __construct(
        private Uuid $orderId,
        private Uuid $clientId,
        /** @var ProductEntry[] */
        private array $products,
    ) {
    }

    public function getOrderId(): Uuid
    {
        return $this->orderId;
    }

    public function setOrderId(Uuid $orderId): void
    {
        $this->orderId = $orderId;
    }

    public function getClientId(): Uuid
    {
        return $this->clientId;
    }

    public function setClientId(Uuid $clientId): void
    {
        $this->clientId = $clientId;
    }

    public function getProducts(): array
    {
        return array_values($this->products);
    }

    public function setProducts(array $products): void
    {
        $this->products = array_combine(
            array_map(fn (ProductEntry $product) => $product->getProductId(), $products),
            array_values($products)
        );
    }

    public function addProduct(ProductEntry $product): void
    {
        $this->products[$product->getProductId()] = $product;
    }

    public function removeProduct(ProductEntry $product): void
    {
        $this->products = array_filter($this->products, fn (ProductEntry $other) => $other->getProductId() != $product->getProductId());
    }

    public function getTotalWeight(): float
    {
        return array_sum(array_map(fn (ProductEntry $product) => $product->getTotalWeight(), $this->products));
    }

    public function getTotalPrice(PricingStrategy $strategy): int
    {
        return $strategy->calculateTotalPriceOfOrder($this);
    }
}
