<?php

namespace App\Controller;

use App\Contract\ClientDto;
use App\Contract\LockClientRequestDto;
use App\Contract\TopUpRequestDto;
use App\Entity\Client;
use App\Exception\ClientAlreadyExistsException;
use App\Service\ClientService;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Component\HttpKernel\Attribute\AsController;
use Symfony\Component\HttpKernel\Attribute\MapRequestPayload;
use Symfony\Component\Routing\Attribute\Route;
use Symfony\Component\Uid\Uuid;

#[Route(path: '/clients', name: 'clients_')]
#[AsController]
readonly class ClientController
{
    public function __construct(
        private ClientService $clientService,
    ) {
    }

    #[Route(name: 'create', methods: ['POST'])]
    public function create(#[MapRequestPayload] ClientDto $clientDto): Response
    {
        $client = new Client(
            clientId: $clientDto->getClientId(),
            name: $clientDto->getName(),
            initialBalance: $clientDto->getBalance(),
        );

        try {
            $this->clientService->createNewClient($client);
        } catch (ClientAlreadyExistsException) {
            return new Response(status: Response::HTTP_CONFLICT);
        }

        return new Response(status: Response::HTTP_CREATED);
    }

    #[Route(path: '/{client}/_top-up', name: 'top_up', methods: ['POST'])]
    public function topUp(
        #[MapRequestPayload] TopUpRequestDto $topUpRequestDto,
        Client $client,
    ): Response {
        $this->clientService->topUpBalance($client, $topUpRequestDto->amount);

        return new Response();
    }

    #[Route(path: '/{client}/_lock', name: 'lock', methods: ['POST'])]
    public function lock(
        #[MapRequestPayload] LockClientRequestDto $lockClientRequestDto,
        Client $client,
    ): Response {
        if ($lockClientRequestDto->lock) {
            $this->clientService->lock($client);
        } else {
            $this->clientService->unlock($client);
        }

        return new Response();
    }
}
